@@ -1,28 +1,25 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Caching helpers."""
 
-import contextlib
 import functools
+from contextlib import contextmanager
 
 from .apidoc import api_end, api_start
 
@@ -37,7 +34,7 @@ CACHED_PREFIX = "_cached_"
 CACHED_PROP_UNLOCKED_TOKEN = "_CACHED_PROPERTIES_ARE_UNLOCKED"
 
 
-@contextlib.contextmanager
+@contextmanager
 def unlocked_cached_properties(obj):
     """Unlock the setters of cached instance attributes.
 
@@ -103,6 +100,8 @@ class cached_property(property):
 
         property.__init__(self, fget, fset, fdel, doc)
 
+        self.__module__ = fget.__module__
+
     def __get__(self, obj, objtype=None):
         if obj is None:
             return self
