@@ -48,11 +48,7 @@ class LMIConstraint(ConicConstraint):
         :param str customString:
             Optional string description.
         """
-        from ..expressions import (AffineExpression, HermitianVariable,
-                                   SymmetricVariable)
-        from ..expressions.vectorizations import (HermitianVectorization,
-                                                  SymmetricVectorization)
-        from ..expressions.data import cvxopt_equals
+        from ..expressions import AffineExpression
 
         required_type = self._required_type()
 
@@ -81,22 +77,6 @@ class LMIConstraint(ConicConstraint):
                 "necessarily {}. Consider a constraint on {} instead.".format(
                 psd.string, needed, glyphs.Tr("{}.hermitianized")(psd.string)))
 
-        # Check if the constraint simply poses positive semidefiniteness on a
-        # matrix variable, as certain solvers can handle this more efficiently
-        # than a general linear matrix inequality.
-        self.semidefVar = None
-        if len(psd._linear_coefs) == 1 and not psd._constant_coef:
-            var, coef = list(psd._linear_coefs.items())[0]
-
-            if isinstance(var, SymmetricVariable) and cvxopt_equals(
-                    coef, SymmetricVectorization(psd.shape).identity,
-                    relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
-                self.semidefVar = var
-            elif isinstance(var, HermitianVariable) and cvxopt_equals(
-                    coef, HermitianVectorization(psd.shape).identity,
-                    relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
-                self.semidefVar = var
-
         super(LMIConstraint, self).__init__(
             self._get_type_term(), customString, printSize=True)
 
@@ -108,6 +88,24 @@ class LMIConstraint(ConicConstraint):
 
         return AffineExpression
 
+    @cached_property
+    def semidefVar(self):
+        """The variable posed positive semidefinite, or :obj:`None`."""
+        from ..expressions import HermitianVariable, SymmetricVariable
+        from ..expressions.data import cvxopt_equals
+
+        psd = self.psd
+
+        if len(psd._linear_coefs) == 1 and not psd._constant_coef:
+            var, coef = next(iter(psd._linear_coefs.items()))
+
+            if isinstance(var, (SymmetricVariable, HermitianVariable)) \
+            and cvxopt_equals(coef, var._vec.identity,
+                    relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
+                return var
+
+        return None
+
     @property
     def smaller(self):
         """The smaller-or-equal side expression."""
