@@ -1,35 +1,33 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Backend for constraint type implementations."""
 
 import random
 import threading
+from abc import ABC, abstractmethod
+from copy import copy as pycopy
 
 import cvxopt
 
 from .. import glyphs
 from ..apidoc import api_end, api_start
 from ..caching import cached_property, empty_cache
-from ..compat import ABC, abstractmethod
 from ..containers import DetailedType
 
 _API_START = api_start(globals())
@@ -173,8 +171,8 @@ class Constraint(ABC):
         """The unique ID of the constraint, assigned at creation.
 
         The ID is kept when the constraint is copied via
-        :meth:`replace_variables`, so that the copy can be identified with the
-        original despite pointing to different expressions and variable objects.
+        :meth:`replace_mutables`, so that the copy can be identified with the
+        original despite pointing to different expressions and mutable objects.
         """
         return self._id
 
@@ -194,6 +192,9 @@ class Constraint(ABC):
         The dimensionality of the constraint, more precisely the dimensionality
         of its Lagrange dual variable, as a pair.
         """
+        # TODO: This seems to be associated with the size of the slack vector
+        #       as well (the test bench checks against this) and therefor should
+        #       probably be abstract as well.
         raise NotImplementedError(
             "{} does not define a constraint (dual value) dimensionality."
             .format(self.__class__.__name__))
@@ -237,28 +238,16 @@ class Constraint(ABC):
         Stores a dual solution value for the dual variable corresponding to the
         constraint in a consistent manner.
 
-        Duals for multidmensional constraints are stored as a
-        :class:`CVXOPT matrix <cvxopt.matrix>` while duals for scalar
-        constraints are stored as a float.
+        Duals for multidmensional constraints are stored as a CVXOPT (sparse)
+        matrix while duals for scalar constraints are stored as a Python scalar.
         """
+        from ..expressions.data import load_data
+
         if value is None:
             self._dual = None
-        elif type(value) in (int, float, complex):
-            if len(self) != 1:
-                raise ValueError("Incompatible dimensions of dual value: "
-                    "Expected {} but got a scalar.".format(self.size))
-
-            self._dual = value
         else:
-            try:
-                dual = cvxopt.matrix(value, self.size)
-            except TypeError:
-                raise ValueError("Incompatible dimensions of dual value for {}:"
-                    " Expected {} and got container of size {}."
-                    .format(self.__class__.__name__, glyphs.size(*self.size),
-                    len(value)))
-
-            self._dual = dual[0] if len(dual) == 1 else dual
+            dual = load_data(value, self.size)[0]
+            self._dual = dual[0] if dual.size == (1, 1) else dual
 
     size  = property(lambda self: self._get_size(), doc=_get_size.__doc__)
     slack = property(lambda self: self._wrap_get_slack(),
@@ -285,9 +274,11 @@ class Constraint(ABC):
             "on selected problems instead.")
 
     # TODO: Solve this with inspection instead of _expression_names?
-    # TODO: Is this even needed for anything apart from variables and
-    #       replace_variables? If not, maybe just implement those two with
+    # TODO: Is this even needed for anything apart from mutables and
+    #       replace_mutables? If not, maybe just implement those two with
     #       every constraint just like for expressions?
+    # Idea: Could add a MutableRegister abstract base class used by Expression
+    #       and Constraint.
     @property
     def expressions(self):
         """Yield expressions stored with the constraint."""
@@ -295,40 +286,54 @@ class Constraint(ABC):
             yield getattr(self, name)
 
     @cached_property
-    def variables(self):
-        """All variables referenced by the constraint."""
-        vars = frozenset()
+    def mutables(self):
+        """All mutables referenced by the constraint."""
+        mtbs = frozenset()
         for expression in self.expressions:
-            vars = vars.union(expression.variables)
-        return vars
+            mtbs = mtbs.union(expression.mutables)
+        return mtbs
 
-    def replace_variables(self, new_variables):
-        """Make the constraint concern a different set of variables.
+    @cached_property
+    def variables(self):
+        """All decision variables referenced by the constraint."""
+        from ..expressions.variables import BaseVariable
 
-        See :meth:`~.expression.Expression.replace_variables` for more.
+        return frozenset(mutable for mutable in self.mutables
+            if isinstance(mutable, BaseVariable))
+
+    @cached_property
+    def parameters(self):
+        """All parameters referenced by the constraint."""
+        from ..expressions.variables import BaseVariable
+
+        return frozenset(mutable for mutable in self.mutables
+            if not isinstance(mutable, BaseVariable))
+
+    def replace_mutables(self, new_mutables):
+        """Make the constraint concern a different set of mutables.
+
+        See :meth:`~.expression.Expression.replace_mutables` for more.
         """
-        from copy import copy
+        the_copy = pycopy(self)
 
-        theCopy = copy(self)
-
-        # TODO: Reset any dual value assigned to the constraint?
-        # theCopy._dual = None
-
-        # Clear the cache of the copy as it can reference old variables.
-        empty_cache(theCopy)
+        # Clear the cache of the copy as it can reference old mutables.
+        empty_cache(the_copy)
 
         # Rewrite expressions.
-        for name in theCopy._expression_names():
+        for name in the_copy._expression_names():
             expression = getattr(self, name)
-            new_expression = expression.replace_variables(new_variables)
-            setattr(theCopy, name, new_expression)
+            new_expression = expression.replace_mutables(new_mutables)
+            setattr(the_copy, name, new_expression)
 
-        # HACK: Delete a custom string because it can contain old variable
-        #       names. In particular Norm uses it when creating a SOCConstraint.
+        # HACK: Delete a custom string because it can contain old mutable names.
+        #       In particular Norm uses it when creating a SOCConstraint.
         # TODO: Get rid of custom strings.
-        theCopy.customString = None
+        the_copy.customString = None
 
-        return theCopy
+        # TODO: Reset a dual value assigned to the constraint?
+        # theCopy._dual = None
+
+        return the_copy
 
     # TODO: Evaluate uses of this method.
     def constring(self):
@@ -368,6 +373,45 @@ class Constraint(ABC):
         return self.relation == self.GE
 
 
+class ConicConstraint(Constraint):
+    """Base class for constraints with an immediate conic representation."""
+
+    @property
+    @abstractmethod
+    def conic_membership_form(self):
+        r"""The constraint in conic membership form.
+
+        For a conic constraint
+        :math:`Ax \succeq_C b \Leftrightarrow Ax - b \in C` this is the pair
+        :math:`(Ax - b, C)` where :math:`Ax - b` is an affine expression and
+        :math:`C` a basic cone supported by PICOS.
+        """
+        pass
+
+    @cached_property
+    def conic_standard_form(self):
+        r"""The constraint in conic standard form.
+
+        For a conic constraint :math:`Ax \succeq_C b` this is the triple
+        :math:`(Ax, b, C)` where :math:`Ax` is a linear expression, :math:`b` is
+        a constant, and :math:`C` a basic cone supported by PICOS.
+        """
+        member, cone = self.conic_membership_form
+        return member.lin, -member.cst, cone
+
+    @cached_property
+    def inverse_conic_standard_form(self):
+        r"""The constraint in inverse conic standard form.
+
+        For a conic constraint
+        :math:`Ax \succeq_C b \Leftrightarrow -Ax \preceq_C -b` this is the
+        triple :math:`(-Ax, -b, C)` where :math:`Ax` is a linear expression,
+        :math:`b` is a constant, and :math:`C` a basic cone supported by PICOS.
+        """
+        member, cone = self.conic_membership_form
+        return -member.lin, member.cst, cone
+
+
 class ConstraintConversion(ABC):
     """Recipe for conversion from one constraint to a set of other constraints.
 
@@ -378,9 +422,9 @@ class ConstraintConversion(ABC):
 
     Implementation class names must end in ``Conversion``, and in particular may
     be called just ``Conversion``. If for instance
-    :class:`AbsoluteValueConstraint <picos.constraints.AbsoluteValueConstraint>`
+    :class:`AbsoluteValueConstraint <.con_absolute.AbsoluteValueConstraint>`
     defines :class:`AffineConversion
-    <picos.constraints.AbsoluteValueConstraint.AffineConversion>`, then the
+    <.con_absolute.AbsoluteValueConstraint.AffineConversion>`, then the
     reformulation will be coined ``AbsoluteValueToAffineReformulation``. If the
     conversions was just named ``Conversion``, the result would be a class named
     ``AbsoluteValueReformulation``.
