@@ -59,6 +59,49 @@ _API_START = api_start(globals())
 # ------------------------------------------------------------------------------
 
 
+@functools.lru_cache()
+def O(rows=1, cols=1):  # noqa
+    """Create a zero matrix.
+
+    :Example:
+
+    >>> from picos import O
+    >>> print(O(2, 3))
+    [0 0 0]
+    [0 0 0]
+    """
+    return AffineExpression.zero((rows, cols))
+
+
+@functools.lru_cache()
+def I(size=1):  # noqa
+    """Create an identity matrix.
+
+    :Example:
+
+    >>> from picos import I
+    >>> print(I(3))
+    [ 1.00e+00     0         0    ]
+    [    0      1.00e+00     0    ]
+    [    0         0      1.00e+00]
+    """
+    return AffineExpression.from_constant("I", (size, size))
+
+
+@functools.lru_cache()
+def J(rows=1, cols=1):
+    """Create a matrix of all ones.
+
+    :Example:
+
+    >>> from picos import J
+    >>> print(J(2, 3))
+    [ 1.00e+00  1.00e+00  1.00e+00]
+    [ 1.00e+00  1.00e+00  1.00e+00]
+    """
+    return AffineExpression.from_constant(1, (rows, cols))
+
+
 def sum(lst, it=None, indices=None):
     """Sum PICOS expressions and give the result a meaningful description.
 
