@@ -1,23 +1,20 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019-2020 Maximilian Stahlberg
 # Based on the original picos.expressions module by Guillaume Sagnol.
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Functions to load and work with raw numeric data."""
@@ -28,13 +25,13 @@ import math
 import re
 import sys
 from fractions import Fraction
+from functools import lru_cache
 
 import cvxopt
 import numpy
 
-from .. import glyphs
+from .. import glyphs, settings
 from ..apidoc import api_end, api_start
-from ..compat import MAJOR, long
 
 _API_START = api_start(globals())
 # -------------------------------
@@ -117,6 +114,17 @@ def should_be_sparse(shape, numNonZero):
     return numNonZero < l * math.log(l)
 
 
+_LOAD_DATA_REGEX = re.compile("^{}{}{}$".format(
+    r"([\-0-9.+j]+)?",  # Leading coefficient.
+    "(" + "|".join((  # Matrix type:
+            r"e_[0-9]+(?:,[0-9]+)?",  # Single nonzero element.
+            r"\|[\-0-9.+j]+\|",  # All equal elements.
+            "I",  # Identity matrix.
+        )) + ")",
+    r"(\([0-9]+(?:,[0-9]+)?\))?"  # Fixed shape.
+))
+
+
 def load_data(value, shape=None, typecode=None, sparse=None,
         alwaysCopy=True, legacy=False):
     r"""Load a constant numeric data value as a CVXOPT (sparse) matrix.
@@ -270,8 +278,8 @@ def load_data(value, shape=None, typecode=None, sparse=None,
     >>> load_data('5.3I', (2,2))
     (<2x2 sparse matrix, tc='d', nnz=2>, '5.3·I')
     """
-    from .expression import Expression
     from .exp_affine import ComplexAffineExpression
+    from .expression import Expression
 
     def load_sparse(V, I, J, shape, typecode):
         """Create a CVXOPT sparse matrix."""
@@ -296,9 +304,9 @@ def load_data(value, shape=None, typecode=None, sparse=None,
                     except Exception:
                         pass
 
-            raise TypeError("Failed to create a CVXOPT sparse matrix of shape "
-                "{} and type '{}': {}.".format(glyphs.shape(shape), typecode,
-                str(error).capitalize()))
+            raise TypeError(
+                "Failed to create a CVXOPT sparse matrix of shape {} and type "
+                "'{}'.".format(glyphs.shape(shape), typecode)) from error
 
     def load_dense(value, shape, typecode):
         """Create a CVXOPT dense matrix."""
@@ -328,9 +336,9 @@ def load_data(value, shape=None, typecode=None, sparse=None,
                                 except Exception:
                                     pass
 
-            raise TypeError("Failed to create a CVXOPT dense matrix of shape "
-                "{} and type '{}': {}.".format(glyphs.shape(shape), typecode,
-                str(error).capitalize()))
+            raise TypeError(
+                "Failed to create a CVXOPT dense matrix of shape {} and type "
+                "'{}'.".format(glyphs.shape(shape), typecode)) from error
 
     def simple_vector_as_row(shape):
         """Whether a single-axis vector should be a row or column vector."""
@@ -392,8 +400,8 @@ def load_data(value, shape=None, typecode=None, sparse=None,
     # Allow conversions to specify their own string description.
     string = None
 
-    # Allow Python 3 range types.
-    if MAJOR >= 3 and isinstance(value, range):
+    # Convert from range to list.
+    if isinstance(value, range):
         value = list(value)
 
     # Try to refine a PICOS expression to a constant affine expression.
@@ -401,11 +409,11 @@ def load_data(value, shape=None, typecode=None, sparse=None,
         value = value.refined
 
     # Convert the data to a CVXOPT (sparse) matrix of proper shape and type.
-    if isinstance(value, (int, long, float, complex, numpy.number)):
+    if isinstance(value, (int, float, complex, numpy.number)):
         # Convert NumPy numbers to Python ones.
         if isinstance(value, numpy.number):
             if isinstance(value, numpy.integer):
-                value = int(value)  # Result can be a Python 2 long.
+                value = int(value)
             elif isinstance(value, numpy.floating):
                 value = float(value)
             elif isinstance(value, numpy.complexfloating):
@@ -415,16 +423,10 @@ def load_data(value, shape=None, typecode=None, sparse=None,
                     type(value).__name__)
 
         # CVXOPT is limited by the system's word length.
-        if isinstance(value, (int, long)):
+        if isinstance(value, int):
             if value > sys.maxsize or value < -sys.maxsize - 1:
-                raise ValueError(
-                    "The number {} is too large to be loaded by PICOS."
-                    .format(value))
-
-            if isinstance(value, long):
-                value = int(value)
-                assert isinstance(value, int), "Failed to convert from long to"\
-                    " int despite being smaller than sys.maxsize."
+                raise ValueError("The number {} is too large to be loaded by "
+                    "PICOS.".format(value))
 
         inShape  = (1, 1)
         outShape = blend_shapes(shape, inShape)
@@ -714,23 +716,17 @@ def load_data(value, shape=None, typecode=None, sparse=None,
         else:
             return load_data(value, shape, typecode, sparse, alwaysCopy, legacy)
 
-        regex = \
-            r"^([\-0-9.+j]+)?" + "(" +\
-            "e_[0-9]+(?:,[0-9]+)?" + "|" +\
-            r"\|[\-0-9.+j]+\|" + "|" +\
-            "I" + ")" +\
-            r"(\([0-9]+(?:,[0-9]+)?\))?$"
+        match = _LOAD_DATA_REGEX.match(value)
 
-        try:
-            tokens = re.findall(regex, value)[0]
-            if len(tokens) != 3:
-                raise ValueError("Unexpected number of tokens.")
-        except Exception:
+        if not match:
             raise ValueError("The string '{}' could not be parsed as a matrix."
                 .format(value))
 
         # Retrieve the string tokens.
+        tokens = match.groups()
+        assert len(tokens) == 3
         factor, base, inShape = tokens
+        assert base is not None
 
         # Convert the factor.
         factor = scalar(factor) if factor else 1
@@ -746,9 +742,9 @@ def load_data(value, shape=None, typecode=None, sparse=None,
             inShape = (int(inShape[0]), int(inShape[1]))
 
             if blend_shapes(shape, inShape) != inShape:
-                raise ValueError("Inconsistent shapes for matrix given as '{}' "
-                    "with expected shape {}."
-                    .format(value, glyphs.shape(shape)))
+                raise ValueError(
+                    "Inconsistent shapes for matrix given as '{}' with expected"
+                    " shape {}.".format(value, glyphs.shape(shape)))
 
             outShape = inShape
         elif None in shape:
@@ -805,11 +801,11 @@ def load_data(value, shape=None, typecode=None, sparse=None,
             value = load_sparse([1.0]*n, range(n), range(n), outShape, typecode)
             string = glyphs.scalar(1) if n == 1 else glyphs.idmatrix()
         else:
-            assert False, "Unexpected matrix base string."
+            assert False, "Unexpected matrix base string '{}'.".format(base)
 
         # Apply a coefficient.
         if factor != 1:
-            value *= factor
+            value = value * factor
             string = glyphs.mul(glyphs.scalar(factor), string)
 
         # Finalize the string.
@@ -932,7 +928,7 @@ def load_dense_data(value, shape=None, typecode=None, alwaysCopy=True):
         alwaysCopy=alwaysCopy)
 
 
-def convert_and_refine_arguments(*which, **kwargs):
+def convert_and_refine_arguments(*which, refine=True, allowNone=False):
     """Convert selected function arguments to PICOS expressions.
 
     If the selected arguments are already PICOS expressions, they are refined
@@ -949,19 +945,11 @@ def convert_and_refine_arguments(*which, **kwargs):
     :param bool allowNone:
         Whether :obj:`None` is passed through to the function.
     """
-    # HACK: Python 2 does not allow individual keyword arguments following
-    #       starred positional ones, so map them manually.
-    refine    = kwargs.pop("refine")    if "refine"    in kwargs else True
-    allowNone = kwargs.pop("allowNone") if "allowNone" in kwargs else False
-    if kwargs:  # Mimic Python 3's usual exception.
-        raise TypeError("'{}' is an invalid keyword argument for "
-            "convert_and_refine_arguments()".format(next(kwargs.keys())))
-
     def decorator(func):
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
-            from .expression import Expression
             from .exp_affine import Constant
+            from .expression import Expression
 
             def name():
                 if hasattr(func, "__qualname__"):
@@ -983,9 +971,10 @@ def convert_and_refine_arguments(*which, **kwargs):
                 else:
                     try:
                         arg = Constant(arg)
-                    except Exception as e:
-                        raise TypeError("Failed to convert argument '{}' of {} "
-                            "to a PICOS constant: {}".format(key, name(), e))
+                    except Exception as error:
+                        raise TypeError(
+                            "Failed to convert argument '{}' of {} to a PICOS "
+                            "constant.".format(key, name())) from error
 
                 newargs[key] = arg
 
@@ -1066,13 +1055,13 @@ def convert_operands(
     def decorator(operator):
         @functools.wraps(operator)
         def wrapper(lhs, rhs, *args, **kwargs):
-            def error(reason="Operand shapes do not match."):
+            def fail(reason="Operand shapes do not match.", error=None):
                 opName  = operator.__qualname__ \
                     if hasattr(operator, "__qualname__") else operator.__name__
                 lhsName = lhs.string if hasattr(lhs, "string") else repr(lhs)
                 rhsName = rhs.string if hasattr(rhs, "string") else repr(rhs)
                 raise TypeError("Invalid operation {}({}, {}): {}".format(
-                    opName, lhsName, rhsName, reason))
+                    opName, lhsName, rhsName, reason)) from error
 
             def make_lhs_shape(rhsShape):
                 if sameShape:
@@ -1106,9 +1095,10 @@ def convert_operands(
                 else:
                     return None
 
-            from .set        import Set
+            from .exp_affine import Constant
+            from .exp_biaffine import BiaffineExpression
             from .expression import Expression
-            from .exp_affine import Constant, ComplexAffineExpression
+            from .set import Set
 
             lhsIsExpOrSet = isinstance(lhs, (Expression, Set))
             rhsIsExpOrSet = isinstance(rhs, (Expression, Set))
@@ -1121,10 +1111,10 @@ def convert_operands(
                     "not pose any shape requirements."
 
             if lhsIsExpOrSet and rhsIsExpOrSet:
-                lhsIsAffine = isinstance(lhs, ComplexAffineExpression)
-                rhsIsAffine = isinstance(rhs, ComplexAffineExpression)
+                lhsIsAffine = isinstance(lhs, BiaffineExpression)
+                rhsIsAffine = isinstance(rhs, BiaffineExpression)
 
-                # If neither expression is affine, it's the operation's job to
+                # If neither expression is biaffine, it's the operation's job to
                 # deal with it.
                 if not lhsIsAffine and not rhsIsAffine:
                     return operator(lhs, rhs, *args, **kwargs)
@@ -1184,8 +1174,8 @@ def convert_operands(
 
                 # At least one of the expressions is affine and we didn't find a
                 # way to fix a detected shape mismatch. It's our job to error.
-                error("The operand shapes of {} and {} do not match."
-                    .format(glyphs.shape(lhsShape), glyphs.shape(rhsShape)))
+                fail("The operand shapes of {} and {} do not match.".format(
+                    glyphs.shape(lhsShape), glyphs.shape(rhsShape)))
             elif lhsIsExpOrSet:
                 if allowNone and rhs is None:
                     return operator(lhs, rhs, *args, **kwargs)
@@ -1204,9 +1194,9 @@ def convert_operands(
                                 rhs = Constant(rhs, shape=rhsShape)
                     else:
                         rhs = Constant(rhs, shape=rhsShape)
-                except Exception as exc:
-                    error("Failed to load right hand side as a constant of "
-                        "shape {}: {}".format(glyphs.shape(rhsShape), exc))
+                except Exception as error:
+                    fail("Could not load right hand side as a constant of "
+                        "matching shape.", error)
             elif rhsIsExpOrSet:
                 if allowNone and lhs is None:
                     return operator(lhs, rhs, *args, **kwargs)
@@ -1225,9 +1215,9 @@ def convert_operands(
                                 lhs = Constant(lhs, shape=lhsShape)
                     else:
                         lhs = Constant(lhs, shape=lhsShape)
-                except Exception as exc:
-                    error("Failed to load left hand side as a constant of "
-                        "shape {}: {}".format(glyphs.shape(rhsShape), exc))
+                except Exception as error:
+                    fail("Could not load left hand side as a constant of "
+                        "matching shape.", error)
             else:
                 assert False, "convert_operands is supposed to decorate " \
                     "expression methods, but neither operand is a PICOS " \
@@ -1238,23 +1228,46 @@ def convert_operands(
     return decorator
 
 
-def cvxopt_equals(A, B, tolerance=None):
-    """Whether two CVXOPT (sparse) matrices are numerically close.
+def cvxopt_equals(A, B, absTol=None, relTol=None):
+    """Whether two CVXOPT (sparse) matrices are numerically equal or close.
 
-    :param float tolerance: Maximum entrywise absolute deviation allowed.
-        Defaults to :data:`TOLERANCE`.
+    For every common entry of ``A`` and ``B``, it is sufficient that one of the
+    two tolerances, ``absTol`` or ``relTol``, is satisfied.
+
+    :param float absTol:
+        Maximum allowed entrywise absolute difference.
+
+    :param float relTol:
+        Maximum allowed entrywise quotient of absolute difference at the entry
+        divided by the largest absolute value of any entry in both matrices.
     """
-    if tolerance is None:
-        tolerance = TOLERANCE
-
     if A.size != B.size:
         return False
 
-    # Work around "ValueError: max() arg is an empty sequence" for sparse zero.
-    if not A and not B:
+    Z = A - B
+
+    if not Z:
         return True
 
-    return max(abs(A - B)) <= tolerance
+    if not absTol and not relTol:
+        return False
+
+    M = max(abs(Z))
+
+    if relTol:
+        N = max(max(abs(A)), max(abs(B)))
+
+    if absTol and relTol:
+        if M > absTol and M / N > relTol:
+            return False
+    elif absTol:
+        if M > absTol:
+            return False
+    else:
+        if M / N > relTol:
+            return False
+
+    return True
 
 
 def cvxopt_maxdiff(A, B):
@@ -1311,17 +1324,89 @@ def cvxopt_vcat(matrices):
 def cvxopt_hpsd(matrix):
     """Whether the given CVXOPT matrix is hermitian positive semidefinite.
 
-    Note that the matrix must be exactly hermitian, no tolerance is applied.
-    """
-    if matrix.size[0] != matrix.size[1]:
-        return False
+    Uses :data:`~picos.settings.RELATIVE_HERMITIANNESS_TOLERANCE` and
+    :data:`~picos.settings.RELATIVE_SEMIDEFINITENESS_TOLERANCE`.
 
-    if not cvxopt_equals(matrix, matrix.H, tolerance=0.0):
+    See also :func:`cvxopt_hpd`.
+
+    .. warning::
+
+        The semidefiniteness tolerance allows negative, near-zero eigenvalues.
+    """
+    if not cvxopt_equals(matrix, matrix.H,
+            relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
         return False
 
     eigenvalues = numpy.linalg.eigvalsh(cvx2np(matrix))
 
-    return all(ev >= 0 for ev in eigenvalues)
+    minimum = -(max(list(eigenvalues) + [1.0])
+        * settings.RELATIVE_SEMIDEFINITENESS_TOLERANCE)
+
+    return all(ev >= minimum for ev in eigenvalues)
+
+
+def cvxopt_hpd(matrix):
+    """Whether the given CVXOPT matrix is hermitian positive definite.
+
+    Uses :data:`~picos.settings.RELATIVE_HERMITIANNESS_TOLERANCE`.
+
+    See also :func:`cvxopt_hpsd`.
+    """
+    if not cvxopt_equals(matrix, matrix.H,
+            relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
+        return False
+
+    eigenvalues = numpy.linalg.eigvalsh(cvx2np(matrix))
+
+    return all(ev > 0 for ev in eigenvalues)
+
+
+def cvxopt_inverse(matrix):
+    """Return the inverse of the given CVXOPT matrix.
+
+    :raises ValueError:
+        If the matrix is not invertible.
+    """
+    matrix_np = cvx2np(matrix)
+
+    try:
+        inverse_np = numpy.linalg.inv(matrix_np)
+    except numpy.linalg.LinAlgError as error:
+        raise ValueError("Failed to invert a {} CVXOPT matrix using NumPy."
+            .format(glyphs.shape(matrix.size))) from error
+
+    inverse, _ = load_data(inverse_np, matrix.size)
+
+    return inverse
+
+
+def cvxopt_principal_root(matrix):
+    """Return the principal square root of a symmetric positive semidef. matrix.
+
+    Given a real symmetric positive (semi)definite CVXOPT input matrix, returns
+    its unique positive (semi)definite matrix square root.
+
+    .. warning::
+
+        Does not validate that the input matrix is symmetric positive
+        semidefinite and will still return a (useless) matrix if it is not.
+    """
+    matrix_np = numpy.array(cvx2np(matrix))  # TODO: GitLab issue #239.
+    U, s, _ = numpy.linalg.svd(matrix_np, hermitian=True)
+    root_np = numpy.dot(U*(s**0.5), U.T)
+    root, _ = load_data(root_np, matrix.size)
+
+    return root
+
+
+@lru_cache()
+def cvxopt_K(m, n, typecode="d"):
+    """The commutation matrix :math:`K_{(m,n)}` as a CVXOPT sparse matrix."""
+    d = m*n
+    V = [1]*d
+    I = range(d)
+    J = [(k % n)*m + k // n for k in I]
+    return cvxopt.spmatrix(V, I, J, (d, d), typecode)
 
 
 def sparse_quadruple(A, reshape=None, preT=False, postT=False):
@@ -1522,7 +1607,7 @@ def value(obj, sparse=None, numpy=False):
 
     :param obj:
         Either a single (PICOS) object that has a ``value`` attribute, such as a
-        :class:`variable <picos.expressions.variables.BaseVariable>`,
+        :class:`mutable <picos.expressions.mutable.Mutable>`,
         :class:`expression <picos.expressions.expression.Expression>` or
         :class:`~picos.modeling.Problem`, or a (nested) :class:`list`,
         :class:`tuple` or :class:`dict` thereof.
