@@ -1,23 +1,20 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg, 2020 Guillaume Sagnol
 # Based on the original picos.expressions module by Guillaume Sagnol.
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implements :class:`Norm`."""
@@ -321,7 +318,7 @@ class Norm(Expression):
     def _get_value(self):
         value = self._x._get_value()
 
-        if len(value) == 1:
+        if value.size == (1, 1):
             return abs(value)
 
         value = cvx2np(value)
@@ -338,9 +335,8 @@ class Norm(Expression):
 
         return cvxopt.matrix(value)
 
-    @cached_unary_operator
-    def _get_variables(self):
-        return self._x.variables
+    def _get_mutables(self):
+        return self._x._get_mutables()
 
     def _is_convex(self):
         return self.p >= 1 or len(self._x) == 1
@@ -348,9 +344,13 @@ class Norm(Expression):
     def _is_concave(self):
         return self.p <= 1 and len(self._x) != 1
 
-    def _replace_variables(self, var_map):
+    def _replace_mutables(self, mapping):
         return self.__class__(
-            self._x._replace_variables(var_map), self.p, self.q, self._limit)
+            self._x._replace_mutables(mapping), self.p, self.q, self._limit)
+
+    def _freeze_mutables(self, freeze):
+        return self.__class__(
+            self._x._freeze_mutables(freeze), self.p, self.q, self._limit)
 
     # --------------------------------------------------------------------------
     # Python special method implementations, except constraint-creating ones.
@@ -387,6 +387,7 @@ class Norm(Expression):
             return NotImplemented
 
     @convert_operands(scalarRHS=True)
+    @refine_operands()
     def __pow__(self, other):
         if isinstance(other, AffineExpression):
             if not other.constant or other.value != 2:
@@ -413,7 +414,7 @@ class Norm(Expression):
     def p(self):
         """The parameter :math:`p`.
 
-        This is a limited precision version of the paramter used when the norm
+        This is a limited precision version of the parameter used when the norm
         was constructed.
         """
         return float(self._pNum) / float(self._pDen)
@@ -432,7 +433,7 @@ class Norm(Expression):
     def q(self):
         """The parameter :math:`q`.
 
-        This is a limited precision version of the paramter used when the norm
+        This is a limited precision version of the parameter used when the norm
         was constructed.
         """
         return float(self._qNum) / float(self._qDen)
@@ -622,9 +623,8 @@ class SpectralNorm(Expression):
         value = numpy.linalg.norm(value, 2)
         return cvxopt.matrix(value)
 
-    @cached_unary_operator
-    def _get_variables(self):
-        return self._x.variables
+    def _get_mutables(self):
+        return self._x._get_mutables()
 
     def _is_convex(self):
         return True
@@ -632,8 +632,11 @@ class SpectralNorm(Expression):
     def _is_concave(self):
         return False
 
-    def _replace_variables(self, var_map):
-        return self.__class__(self._x._replace_variables(var_map))
+    def _replace_mutables(self, mapping):
+        return self.__class__(self._x._replace_mutables(mapping))
+
+    def _freeze_mutables(self, freeze):
+        return self.__class__(self._x._freeze_mutables(freeze))
 
     # --------------------------------------------------------------------------
     # Python special method implementations, except constraint-creating ones.
@@ -807,9 +810,8 @@ class NuclearNorm(Expression):
         value = numpy.linalg.norm(value, 'nuc')
         return cvxopt.matrix(value)
 
-    @cached_unary_operator
-    def _get_variables(self):
-        return self._x.variables
+    def _get_mutables(self):
+        return self._x._get_mutables()
 
     def _is_convex(self):
         return True
@@ -817,8 +819,11 @@ class NuclearNorm(Expression):
     def _is_concave(self):
         return False
 
-    def _replace_variables(self, var_map):
-        return self.__class__(self._x._replace_variables(var_map))
+    def _replace_mutables(self, mapping):
+        return self.__class__(self._x._replace_mutables(mapping))
+
+    def _freeze_mutables(self, freeze):
+        return self.__class__(self._x._freeze_mutables(freeze))
 
     # --------------------------------------------------------------------------
     # Python special method implementations, except constraint-creating ones.
