@@ -1,23 +1,20 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 # Based on the original picos.expressions module by Guillaume Sagnol.
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implements :class:`QuadraticExpression`."""
@@ -26,6 +23,7 @@ import operator
 import sys
 from collections import OrderedDict as odict
 from collections import namedtuple
+from types import MappingProxyType
 
 import cvxopt
 import cvxopt.cholmod
@@ -39,6 +37,7 @@ from ..caching import (borrow_cache, cached_property,
 from ..constraints import (ConicQuadraticConstraint, Constraint,
                            ConvexQuadraticConstraint,
                            NonconvexQuadraticConstraint)
+from ..legacy import deprecated
 from .data import convert_operands, cvx2np, should_be_sparse
 from .exp_affine import AffineExpression, ComplexAffineExpression, Constant
 from .expression import Expression, refine_operands, validate_prediction
@@ -220,7 +219,7 @@ class QuadraticExpression(Expression):
         except ValueError:  # No quadratic part.
             rank = 0
 
-        factors = len(set(self.scalar_factors))
+        factors = len(set(self._sf)) if self._sf else 0
 
         return self.Subtype(convex, concave, qterms, rank, haslin, factors)
 
@@ -236,8 +235,8 @@ class QuadraticExpression(Expression):
         return value
 
     @cached_unary_operator
-    def _get_variables(self):
-        return self._aff.variables.union(
+    def _get_mutables(self):
+        return self._aff._get_mutables().union(
             var for vars in self._quads for var in vars)
 
     def _is_convex(self):
@@ -260,25 +259,30 @@ class QuadraticExpression(Expression):
         else:
             return True
 
-    def _replace_variables(self, var_map):
-        name_map = {old.name: new.name for old, new in var_map.items()}
+    def _replace_mutables(self, mapping):
+        name_map = {old.name: new.name for old, new in mapping.items()}
 
         string = self.string
         if isinstance(string, glyphs.GlStr):
             string = string.reglyphed(name_map)
 
-        quads = {(var_map[var1], var_map[var2]): coef
+        quads = {(mapping[var1], mapping[var2]): coef
             for (var1, var2), coef in self._quads.items()}
 
         if not self._sf:
             sf = None
         elif self._sf[0] is self._sf[1]:
-            sf = (self._sf[0]._replace_variables(var_map),)*2
+            sf = (self._sf[0]._replace_mutables(mapping),)*2
         else:
-            sf = tuple(f._replace_variables(var_map) for f in self._sf)
+            sf = tuple(f._replace_mutables(mapping) for f in self._sf)
 
         return QuadraticExpression(
-            string, quads, self.aff._replace_variables(var_map), sf)
+            string, quads, self.aff._replace_mutables(mapping), sf)
+
+    def _freeze_mutables(self, freeze):
+        # TODO: Allow freezing of quadratic expressions.
+        raise NotImplementedError(
+            "Partially freezing quadratic expressions is not yet supported.")
 
     # --------------------------------------------------------------------------
     # Python special method implementations, except constraint-creating ones.
@@ -431,11 +435,6 @@ class QuadraticExpression(Expression):
         else:
             return NotImplemented
 
-    def __div__(self, other):
-        """Denote division by a scalar constant."""
-        # Python 2 uses this instead of __truediv__.
-        return self.__truediv__(other)
-
     # --------------------------------------------------------------------------
     # Decomposition related methods.
     # --------------------------------------------------------------------------
@@ -459,7 +458,7 @@ class QuadraticExpression(Expression):
 
         # For the augmented matrix, find additional linear occurences.
         if augmentedMatrix:
-            for x, a in self._aff._sparse_coefs.items():
+            for x, a in self._aff._sparse_linear_coefs.items():
                 nonzero.setdefault(x, set())
                 nonzero[x].update(a.I)
 
@@ -483,14 +482,14 @@ class QuadraticExpression(Expression):
 
         # Turn Q into A if requested.
         if augmentedMatrix:
-            for x, a in self._aff._sparse_coefs.items():
+            for x, a in self._aff._sparse_linear_coefs.items():
                 b = a[nonzero[x]]
 
                 V.extend(b.V)
                 I.extend(offsets[x] + i for i in b.I)
                 J.extend([offset]*len(b.J))
 
-            V.append(self._aff._const[0])
+            V.append(self._aff._constant_coef[0])
             I.append(offset)
             J.append(offset)
 
@@ -823,9 +822,15 @@ class QuadraticExpression(Expression):
     # --------------------------------------------------------------------------
 
     @property
+    @deprecated("2.2", "This property will be removed in a future release.")
     def quadratic_forms(self):
-        """The quadratic forms as sparse matrices."""
-        return dict(self._sparse_quads)
+        """The quadratic forms as a map from variable pairs to sparse matrices.
+
+        .. warning::
+
+            Do not modify the returned matrices.
+        """
+        return MappingProxyType(self._sparse_quads)
 
     @property
     def is0(self):
@@ -837,15 +842,12 @@ class QuadraticExpression(Expression):
         r"""Decomposition into scalar real affine expressions.
 
         If the expression is known to be equal to :math:`ab` for scalar real
-        affine expressions :math:`a` and :math:`b`, returns the pair ``(a, b)``.
-        Otherwise, returns the empty tuple ``()``.
+        affine expressions :math:`a` and :math:`b`, this is the pair ``(a, b)``.
+        Otherwise, this is :obj:`None`.
 
         Note that if :math:`a = b`, then also ``a is b`` in the returned tuple.
         """
-        if self._sf:
-            return tuple(self._sf)
-        else:
-            return ()
+        return self._sf
 
     # --------------------------------------------------------------------------
     # Methods and properties that return modified copies.
@@ -867,14 +869,12 @@ class QuadraticExpression(Expression):
     @cached_property
     def lin(self):
         """Linear part :math:`a^Tx` of the quadratic expression."""
-        return AffineExpression(glyphs.linpart(self._symbStr),
-            coefficients=self._aff._coefs)
+        return self._aff.lin.renamed(glyphs.linpart(self._symbStr))
 
     @cached_property
     def cst(self):
         """Constant part :math:`b` of the quadratic expression."""
-        return AffineExpression(glyphs.cstpart(self._symbStr),
-            constant=self._aff._const)
+        return self._aff.cst.renamed(glyphs.cstpart(self._symbStr))
 
     # --------------------------------------------------------------------------
     # Constraint-creating operators, and _predict.
