@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """String templates used to print (algebraic) expressions.
@@ -79,6 +76,7 @@ You can reset all glyphs to their initial state as follows:
 import functools
 import sys
 
+from . import settings
 from .apidoc import api_end, api_start
 
 # Allow functions to modify this module directly.
@@ -393,6 +391,7 @@ ncnorm   = Op(Gl("{}_{}")(glyphs.norm("{}"), "*"), 1, closed=True)
 
 # Function glyphs.
 sum      = Fn("sum({})");   """Summation glyph."""
+prod     = Fn("prod({})");   """Product glyph."""
 max      = Fn("max({})");   """Maximum glyph."""
 min      = Fn("min({})");   """Minimum glyph."""
 exp      = Fn("exp({})");   """Exponentiation glyph."""
@@ -400,9 +399,11 @@ log      = Fn("log({})");   """Logarithm glyph."""
 vec      = Fn("vec({})");   """Vectorization glyph."""
 trilvec  = Fn("trilvec({})");  """Lower triangular vectorization glyph."""
 triuvec  = Fn("triuvec({})");  """Upper triangular vectorization glyph."""
+svec     = Fn("svec({})");  """Symmetric vectorization glyph."""
+desvec   = Fn("desvec({})");   """Symmetric de-vectorization glyph."""
 trace    = Fn("tr({})");    """Matrix trace glyph."""
 diag     = Fn("diag({})");  """Diagonal matrix glyph."""
-maindiag = Fn("maindiag({})");  """Main diagonal glyph."""
+maindiag = Fn("maindiag({})"); """Main diagonal glyph."""
 det      = Fn("det({})");   """Determinant glyph."""
 real     = Fn("Re({})");    """Real part glyph."""
 imag     = Fn("Im({})");    """Imaginary part glyph."""
@@ -410,12 +411,17 @@ conj     = Fn("conj({})");  """Complex conugate glyph."""
 quadpart = Fn("quad({})");  """Quadratic part glyph."""
 affpart  = Fn("aff({})");   """Affine part glyph."""
 linpart  = Fn("lin({})");   """Linear part glyph."""
+blinpart = Fn("bilin({})"); """Bilinear part glyph."""
+ncstpart = Fn("noncst({})"); """Nonconstant part glyph."""
 cstpart  = Fn("cst({})");   """Constant part glyph."""
-value    = Fn("value({})"); """Value of expression glyph."""
-reshaped = Fn("reshaped({}, {})"); """Reshaped glyph."""
-bcasted  = Fn("bcasted({}, {})");  """Broadcasted glyph."""
+frozen   = Fn("[{}]");      """Frozen mutables glyph."""
+reshaped = Fn("reshaped({}, {})"); """Column-major (Fortran) reshaped glyph."""
+reshaprm = Fn("reshaped({}, {}, C)"); """Row-major (C-order) reshaped glyph."""
+bcasted  = Fn("bcasted({}, {})"); """Broadcasted glyph."""
 sqrt     = Fn("sqrt({})");  """Square root glyph."""
-shuffled = Fn("shuffled({})");     """Matrix reshuffling glyph."""
+shuffled = Fn("shuffled({})"); """Matrix reshuffling glyph."""
+probdist = Fn("pd({})");    """Probability distribution glyph."""
+expected = Fn("E[{}]");     """Epected value glyph."""
 
 # Semi-closed glyphs.
 ptrace   = Op("trace_{}({})", 0, closed=[False, True])
@@ -426,16 +432,10 @@ ptransp_ = Op("{}.{{{{{}}}}}", 1, closed=[False, True])
 """Matrix partial transposition glyph."""  # TODO: Replace ptransp.
 ptrace_  = Op("{}.{{{{{}}}}}", 1, closed=[False, True])
 """Matrix partial trace glyph."""  # TODO: Replace ptrace_.
+exparg   = Op("E_{}({})", 0, closed=[False, True])
+"""Expected value glyph."""
 
-# Trailer glyphs.
-power    = Tr("{}^{}"); """Power glyph."""
-cubed    = Tr(glyphs.power("{}", "3")); """Cubed value glyph."""
-squared  = Tr(glyphs.power("{}", "2")); """Squared value glyph."""
-transp   = Tr("{}.T"); """Matrix transposition glyph."""
-ptransp  = Tr("{}.Tx"); """Matrix partial transposition glyph."""
-htransp  = Tr("{}.H"); """Matrix hermitian transposition glyph."""
-
-# Other operator glyphs.
+# Basic algebraic glyphs.
 add      = Op("{} + {}", 3, assoc=True);  """Addition glyph."""
 sub      = Op("{} - {}", 3, assoc=False); """Substraction glyph."""
 hadamard = Op("{}(o){}", 2, assoc=True);  """Hadamard product glyph."""
@@ -445,6 +445,16 @@ div      = Op("{}/{}",   2, assoc=False); """Division glyph."""
 neg      = Op("-{}",     2.5);              """Negation glyph."""
 plsmns   = Op("[+/-]{}", 2.5);              """Plus/Minus glyph."""
 
+# Trailer glyphs.
+power    = Tr("{}^{}"); """Power glyph."""
+cubed    = Tr(glyphs.power("{}", "3")); """Cubed value glyph."""
+squared  = Tr(glyphs.power("{}", "2")); """Squared value glyph."""
+inverse  = Tr(glyphs.power("{}", glyphs.neg(1))); """Matrix inverse glyph."""
+transp   = Tr("{}.T"); """Matrix transposition glyph."""
+ptransp  = Tr("{}.Tx"); """Matrix partial transposition glyph."""
+htransp  = Tr("{}.H"); """Matrix hermitian transposition glyph."""
+index    = Tr("{}_{}"); """Index glyph."""
+
 # Concatenation glyphs.
 horicat  = Op("{}, {}", 4, assoc=True); """Horizontal concatenation glyph."""
 vertcat  = Op("{}; {}", 4.5, assoc=True); """Vertical concatenation glyph."""
@@ -459,6 +469,10 @@ lt       = Rl("{} < {}");  """Lesser than glyph."""
 psdge    = Rl("{} >> {}"); """Lesser or equal w.r.t. the p.s.d. cone glyph."""
 psdle    = Rl("{} << {}"); """Greater or equal w.r.t. the p.s.d. cone glyph."""
 
+# Bracket-less function glyphs.
+maxarg   = Op("max_{} {}", 3.5, closed=False)
+minarg   = Op("min_{} {}", 3.5, closed=False)
+
 
 def latin1(rebuildDerivedGlyphs=True):
     """Let PICOS create future strings using only ISO 8859-1 characters."""
@@ -505,6 +519,7 @@ def unicode(rebuildDerivedGlyphs=True):
     glyphs.leadsto.template  = "{} → {}"
     glyphs.norm.template     = "‖{}‖"
     glyphs.or_.template      = "{} ∨ {}"
+    glyphs.prod.template     = "∏({})"
     glyphs.psdge.template    = "{} ≽ {}"
     glyphs.psdle.template    = "{} ≼ {}"
     glyphs.sum.template      = "∑({})"
@@ -515,8 +530,16 @@ def unicode(rebuildDerivedGlyphs=True):
         rebuild()
 
 
-# Default to unicode glyphs.
-default = unicode
+# Set and use the default charset.
+if settings.DEFAULT_CHARSET == "unicode":
+    default = unicode
+elif settings.DEFAULT_CHARSET == "latin1":
+    default = latin1
+elif settings.DEFAULT_CHARSET == "ascii":
+    default = ascii
+else:
+    raise ValueError("PICOS doesn't have a charset named '{}'.".format(
+        settings.DEFAULT_CHARSET))
 default()
 
 
