@@ -25,8 +25,9 @@ _API_START = api_start(globals())
 
 
 from .footprint  import Footprint, Specification  # noqa
+from .objective  import Objective  # noqa
 from .options    import Options  # noqa
-from .problem    import Objective, Problem, SolutionFailure  # noqa
+from .problem    import Problem, SolutionFailure  # noqa
 from .solution   import Solution  # noqa
 from .strategy   import NoStrategyFound, Strategy  # noqa
 from .quicksolve import find_assignment, maximize, minimize  # noqa
