@@ -370,6 +370,10 @@ OPTIONS = [
         For example, ``{"mip.limits.cutpasses": 5}`` limits the number of
         cutting plane passes when solving the root node to :math:`5`."""),
 
+    ("cplex_vmconfig", str, None, """
+        Load a CPLEX virtual machine configuration file.
+        """, None),
+
     ("cplex_lwr_bnd_limit", float, None, """
         Tells CPLEX to stop MIP optimization if a lower bound below this value
         is found.
@@ -422,6 +426,12 @@ OPTIONS = [
         See the `list of MOSEK (Optimizer) 8.1 parameters
         <https://docs.mosek.com/8.1/pythonapi/parameters.html>`_."""),
 
+    ("mosek_server", str, None, """
+        Address of a MOSEK remote optimization server to use.
+
+        This option affects both MOSEK (Optimizer) and MOSEK (Fusion).
+        """, None),
+
     ("mskfsn_params", dict, {}, """
         A dictionary of MOSEK (Fusion) parameters to be set after general
         options are passed and before the search is started.
