@@ -30,17 +30,17 @@ from types import MappingProxyType
 import cvxopt as cvx
 import numpy as np
 
-from .. import constraints, expressions, glyphs
+from .. import constraints, expressions, glyphs, settings
 from ..apidoc import api_end, api_start
-from ..caching import cached_property
 from ..expressions.data import cvx2np
-from ..expressions.uncertain import IntractableWorstCase, UncertainExpression
+from ..expressions.uncertain import IntractableWorstCase
 from ..expressions.variables import BaseVariable
 from ..formatting import natsorted, parameterized_string, picos_box
 from ..legacy import deprecated, map_legacy_options, throw_deprecation_warning
 from ..solvers import Solver, get_solver
 from .file_out import write
 from .footprint import Footprint, Specification
+from .objective import Objective
 from .options import Options
 from .solution import SS_OPTIMAL, Solution
 
@@ -70,228 +70,6 @@ class SolutionFailure(RuntimeError):
         return "Code {}: {}".format(self.code, self.message)
 
 
-class Objective:
-    """An optimization objective composed of search direction and function."""
-
-    #: Short string denoting a feasibility problem.
-    FIND = "find"
-
-    #: Short string denoting a minimization problem.
-    MIN  = "min"
-
-    #: Short string denoting a maximization problem.
-    MAX  = "max"
-
-    def __init__(self, direction=None, function=None):
-        """Construct an optimization objective.
-
-        :param str direction:
-            Case insensitive search direction string. One of
-
-            - ``"min"`` or ``"minimize"``,
-            - ``"max"`` or ``"maximize"``,
-            - ``"find"`` or :obj:`None` (for a feasibility problem).
-
-        :param ~picos.expressions.Expression function:
-            The objective function. Must be :obj:`None` for a feasibility
-            problem.
-        """
-        if direction is None:
-            direction = self.FIND
-        else:
-            if not isinstance(direction, str):
-                raise TypeError("Search direction must be given as a string.")
-
-            lower = direction.lower()
-            if lower == "find":
-                direction = self.FIND
-            elif lower.startswith("min"):
-                direction = self.MIN
-            elif lower.startswith("max"):
-                direction = self.MAX
-            else:
-                raise ValueError(
-                    "Invalid search direction '{}'.".format(direction))
-
-        if function is None:
-            if direction != self.FIND:
-                raise ValueError("Missing an objective function.")
-        else:
-            if direction == self.FIND:
-                raise ValueError("May not specify an objective function for a "
-                    "feasiblity problem.")
-
-            if not isinstance(function, expressions.Expression):
-                raise TypeError(
-                    "Objective function must be a PICOS expression.")
-
-            if len(function) != 1:
-                raise TypeError("Objective function must be scalar.")
-
-            function = function.refined
-
-            if isinstance(function, expressions.ComplexAffineExpression) \
-            and function.complex:
-                raise TypeError("Objective function may not be complex.")
-
-        self._direction = direction
-        self._function = function
-
-    def __str__(self):
-        if self._function is None:
-            return "find an assignment"
-        else:
-            minimize = self._direction == self.MIN
-            dir_str = "minimize" if minimize else "maximize"
-
-            if self._function.uncertain:
-                obj_str = self._function.worst_case_string(
-                    "max" if minimize else "min")
-            else:
-                obj_str = self._function.string
-
-            return "{} {}".format(dir_str, obj_str)
-
-    def __repr__(self):
-        return glyphs.repr1("Objective: {}".format(self))
-
-    def __iter__(self):
-        yield self._direction
-        yield self._function
-
-    def __eq__(self, other):
-        if not isinstance(other, Objective):
-            return False
-
-        if self._direction != other._direction:
-            return False
-
-        if self._direction == self.FIND:
-            return True
-
-        try:
-            return self._function.equals(other._function)
-        except AttributeError:
-            # TODO: Allow all expressions to be equality-checked?
-            return self._function is other._function
-
-    @property
-    def feasibility(self):
-        """Whether the objective is "find an assignment"."""
-        return self._function is None
-
-    @property
-    def pair(self):
-        """Search direction and objective function as a pair."""
-        return self._direction, self._objective
-
-    @property
-    def direction(self):
-        """Search direction as a short string."""
-        return self._direction
-
-    @property
-    def function(self):
-        """Objective function."""
-        return self._function
-
-    @cached_property
-    def normalized(self):
-        """The objective but with feasiblity posed as "minimize 0".
-
-        >>> from picos import Objective
-        >>> obj = Objective(); obj
-        <Objective: find an assignment>
-        >>> obj.normalized
-        <Objective: minimize 0>
-        """
-        if self._function is None:
-            return Objective(self.MIN, expressions.AffineExpression.zero())
-        else:
-            return self
-
-    @property
-    def value(self):
-        """Value of the objective function.
-
-        In the case of an uncertain objective, this is the worst-case (expected)
-        objective value.
-
-        :raises picos.uncertain.IntractableWorstCase:
-            When computing the worst-case (expected) value of an uncertain
-            objective is not supported.
-        """
-        if self._function is None:
-            return None
-        elif isinstance(self._function, UncertainExpression):
-            if self._direction == self.MIN:
-                bad_direction = self.MAX
-            elif self._direction == self.MAX:
-                bad_direction = self.MIN
-            else:
-                bad_direction = self.FIND
-
-            try:
-                return self._function.worst_case_value(bad_direction)
-            except IntractableWorstCase as error:
-                raise IntractableWorstCase("Failed to compute the worst-case "
-                    "value of the objective function {}: {} Maybe evaluate the "
-                    "nominal objective function instead?"
-                    .format(self._function.string, error)) from None
-        else:
-            return self._function.value
-
-    def __index__(self):
-        if self._function is None:
-            raise TypeError("A feasiblity objective cannot be used as an index "
-                "because there is no objective function to take the value of.")
-
-        value = self.value
-
-        if value is None:
-            raise expressions.NotValued(
-                "Cannot use unvalued objective function {} as an index."
-                .format(self._function.string))
-
-        assert isinstance(value, (float, int))
-
-        if not value.is_integer():
-            raise ValueError(
-                "Cannot use the objective function {} as an index as its value "
-                "of {} is not integral.".format(self._function.string, value))
-
-        return int(value)
-
-    def _casting_helper(self, theType):
-        assert theType in (int, float, complex)
-
-        if self._function is None:
-            raise TypeError("A feasiblity objective cannot be cast as {} "
-                "because there is no objective function to take the value of."
-                .format(theType.__name__))
-
-        value = self.value
-
-        if value is None:
-            raise expressions.NotValued(
-                "Cannot cast unvalued objective function {} as {}."
-                .format(self._function.string, theType.__name__))
-
-        return theType(value)
-
-    def __int__(self):
-        return self._casting_helper(int)
-
-    def __float__(self):
-        return self._casting_helper(float)
-
-    def __complex__(self):
-        return self._casting_helper(complex)
-
-    def __round__(self, ndigits=None):
-        return round(float(self), ndigits)
-
-
 class Problem():
     """PICOS' representation of an optimization problem.
 
@@ -503,16 +281,62 @@ class Problem():
 
     @property
     def no(self):
-        """Normalized objective as an :class:`Objective` instance.
+        """Normalized objective as an :class:`~picos.Objective` instance.
 
         Either a minimization or a maximization objective, with feasibility
         posed as "minimize 0".
 
-        The same as the :attr:`~Objective.normalized` attribute of the
-        :attr:`objective`.
+        The same as the :attr:`~.objective.Objective.normalized` attribute of
+        the :attr:`objective`.
         """
         return self._objective.normalized
 
+    @property
+    def minimize(self):
+        """Minimization objective as an :class:`~.expression.Expression`.
+
+        This can be used to set a minimization objective. For querying the
+        objective, it is recommended to use :attr:`objective` instead.
+        """
+        if self._objective.direction == Objective.MIN:
+            return self._objective.function
+        else:
+            raise ValueError("Objective direction is not minimize.")
+
+    @minimize.setter
+    def minimize(self, value):
+        self.objective = "min", value
+
+    @minimize.deleter
+    def minimize(self):
+        if self._objective.direction == Objective.MIN:
+            del self.objective
+        else:
+            raise ValueError("Objective direction is not minimize.")
+
+    @property
+    def maximize(self):
+        """Maximization objective as an :class:`~.expression.Expression`.
+
+        This can be used to set a maximization objective. For querying the
+        objective, it is recommended to use :attr:`objective` instead.
+        """
+        if self._objective.direction == Objective.MAX:
+            return self._objective.function
+        else:
+            raise ValueError("Objective direction is not maximize.")
+
+    @maximize.setter
+    def maximize(self, value):
+        self.objective = "max", value
+
+    @maximize.deleter
+    def maximize(self):
+        if self._objective.direction == Objective.MAX:
+            del self.objective
+        else:
+            raise ValueError("Objective direction is not maximize.")
+
     @property
     def value(self):
         """Objective function value.
@@ -934,6 +758,14 @@ class Problem():
     def __round__(self, ndigits=None):
         return self._objective.__round__(ndigits)
 
+    def __iadd__(self, constraints):
+        if isinstance(constraints, tuple):
+            self.require(*constraints)
+        else:
+            self.require(constraints)
+
+        return self
+
     # --------------------------------------------------------------------------
     # Bookkeeping methods.
     # --------------------------------------------------------------------------
@@ -1149,12 +981,22 @@ class Problem():
             return self._constraints[idOrIds]
 
     def add_constraint(self, constraint, key=None):
-        """Add a constraint to the problem.
+        """Add a single constraint to the problem and return it.
 
-        :param constraint: The constraint to be added.
-        :type constraint: :class:`Constraint <picos.constraints.Constraint>`
-        :param str key: Optional name of the constraint.
-        :returns: The constraint that was added to the problem.
+        :param constraint:
+            The constraint to be added.
+        :type constraint:
+            :class:`Constraint <picos.constraints.Constraint>`
+
+        :param key: DEPRECATED
+
+        :returns:
+            The constraint that was added to the problem.
+
+        .. note::
+
+            This method is superseded by the more compact and more flexible
+            :meth:`require` method or, at your preference, the ``+=`` operator.
         """
         # Handle deprecated 'key' parameter.
         if key is not None:
@@ -1171,17 +1013,17 @@ class Problem():
         return constraint
 
     def add_list_of_constraints(self, lst, it=None, indices=None, key=None):
-        """Add a list of constraints to the problem.
+        """Add constraints from an iterable to the problem.
 
-        Adds a list of constraints to the problem, enabling the use of
-        Python list comprehensions (see the example below).
+        :param lst:
+            Iterable of constraints to add.
 
-        :param list(picos.constraints.Constraint) lst: Constraints to add.
         :param it: DEPRECATED
         :param indices: DEPRECATED
         :param key: DEPRECATED
 
-        :returns: A list of all constraints that were added.
+        :returns:
+            A list of all constraints that were added.
 
         :Example:
 
@@ -1217,6 +1059,11 @@ class Problem():
             uᵀ·x[i] ≤ y[i] ∀ i ∈ [0…4]
             ‖w[i,j]‖ ≤ y[j] ∀ (i,j) ∈ zip([1,2,4],[2,0,2])
             y[i] ≥ y[i+1] ∀ i ∈ [0…3]
+
+        .. note::
+
+            This method is superseded by the more compact and more flexible
+            :meth:`require` method or, at your preference, the ``+=`` operator.
         """
         if it is not None or indices is not None or key is not None:
             # Deprecated as of 2.0.
@@ -1233,8 +1080,66 @@ class Problem():
 
         return added
 
-    # TODO: Add Problem.require replacing add_constraint and
-    #       add_list_of_constraints?
+    def require(self, *constraints, ret=False):
+        """Add constraints to the problem.
+
+        :param constraints:
+            A sequence of constraints or constraint groups (iterables yielding
+            constraints) or a mix thereof.
+
+        :param bool ret:
+            Whether to return the added constraints.
+
+        :returns:
+            When ``ret=True``, returns either the single constraint that was
+            added, the single group of constraint that was added in the form of
+            a :class:`list` or, when multiple arguments are given, a list of
+            constraints or constraint groups represented as above. When
+            ``ret=False``, returns nothing.
+
+        :Example:
+
+        >>> from picos import Problem, RealVariable
+        >>> x = RealVariable("x", 5)
+        >>> P = Problem()
+        >>> P.require(x >= -1, x <= 1)  # Add individual constraints.
+        >>> P.require([x[i] <= x[i+1] for i in range(4)])  # Add groups.
+        >>> print(P)
+        Linear Feasibility Problem
+          find an assignment
+          for
+            5×1 real variable x
+          subject to
+            x ≥ [-1]
+            x ≤ [1]
+            x[i] ≤ x[i+1] ∀ i ∈ [0…3]
+
+        .. note::
+
+            For a single constraint ``C``, ``P.require(C)`` may also be written
+            as ``P += C``. For multiple constraints, ``P.require([C1, C2])`` can
+            be abbreviated ``P += [C1, C2]`` while ``P.require(C1, C2)`` can be
+            written as either ``P += (C1, C2)`` or just ``P += C1, C2``.
+        """
+        from ..constraints import Constraint
+
+        added = []
+        for constraint in constraints:
+            if isinstance(constraint, Constraint):
+                added.append(self.add_constraint(constraint))
+            else:
+                try:
+                    if not all(isinstance(c, Constraint) for c in constraint):
+                        raise TypeError
+                except TypeError:
+                    raise TypeError(
+                        "An argument is neither a constraint nor an iterable "
+                        "yielding constraints.") from None
+                else:
+                    added.append(self.add_list_of_constraints(constraint))
+
+        if ret:
+            return added[0] if len(added) == 1 else added
 
     def _con_group_index(self, conOrConID):
         """Support :meth:`remove_constraint`."""
@@ -1954,53 +1859,8 @@ class Problem():
                 print("Search {:.1e}s, solve {:.1e}s, overhead {:.0%}."
                     .format(searchTime, solveTime, overhead))
 
-        return solutions
-
-    @deprecated("2.0", reason="Misleading semantics. Maybe "
-        ":func:`picos.minimize` is what you want.")
-    def minimize(self, obj, **extra_options):
-        """Look for a minimizing solution.
-
-        Sets the objective to minimize the given objective function and calls
-        the solver with the given additional options.
-
-        :param obj: The objective function to minimize.
-        :type obj: :class:`~picos.expressions.Expression`
-        :param extra_options: A sequence of additional solver options.
-
-        :returns: A dictionary, see :meth:`~Problem.solve`.
-
-        .. warning::
-
-            This is equivalent to :meth:`~Problem.set_objective`
-            followed by :meth:`~Problem.solve` and will thus override
-            any existing objective function and direction.
-        """
-        self.objective = "min", obj
-        return self.solve(**extra_options)
-
-    @deprecated("2.0", reason="Misleading semantics. Maybe "
-        ":func:`picos.maximize` is what you want.")
-    def maximize(self, obj, **extra_options):
-        """Look for a maximization solution.
-
-        Sets the objective to maximize the given objective function and calls
-        the solver with the given additional options.
-
-        :param obj: The objective function to maximize.
-        :type obj: :class:`~picos.expressions.Expression`
-        :param extra_options: A sequence of additional solver options.
-
-        :returns: A dictionary, see :meth:`~Problem.solve`.
-
-        .. warning::
-
-            This is equivalent to :meth:`~Problem.set_objective`
-            followed by :meth:`~Problem.solve` and will thus override
-            any existing objective function and direction.
-        """
-        self.objective = "max", obj
-        return self.solve(**extra_options)
+        if settings.RETURN_SOLUTION:
+            return solutions
 
     def write_to_file(self, filename, writer="picos"):
         """See :func:`picos.modeling.file_out.write`."""
