@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`ECOSSolver`."""
@@ -24,9 +21,8 @@
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
-from ..constraints import (AffineConstraint, ExpConeConstraint, RSOCConstraint,
-                           SOCConstraint)
+from ..constraints import (AffineConstraint, DummyConstraint,
+                           ExpConeConstraint, RSOCConstraint, SOCConstraint)
 from ..expressions import AffineExpression, BinaryVariable, IntegerVariable
 from ..modeling.footprint import Specification
 from ..modeling.solution import (PS_FEASIBLE, PS_INFEASIBLE, PS_UNBOUNDED,
@@ -42,24 +38,30 @@ _API_START = api_start(globals())
 class ECOSSolver(Solver):
     """Interface to the ECOS solver via its official Python interface."""
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
             ExpConeConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -69,13 +71,18 @@ class ECOSSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("ecos")
+        cls.check_import("ecos")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "ecos", "ECOS", "Embedded Conic Solver"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return True
+
     def __init__(self, problem):
         """Initialize an ECOS solver interface.
 
@@ -105,8 +112,6 @@ class ECOSSolver(Solver):
         self._ecosVarOffset.clear()
         self._ecosConIndices.clear()
 
-        # self._objectiveOffset = 0.0
-
     @property
     def ecos(self):
         """Return the ECOS core module ecos.py.
@@ -159,7 +164,7 @@ class ECOSSolver(Solver):
 
         # Construct G.
         I, J, V = [], [], []
-        for variable, coefficients in expression._coefs.items():
+        for variable, coefficients in expression._linear_coefs.items():
             ecosVarOffset = self._ecosVarOffset[variable]
 
             if not isinstance(coefficients, cvxopt.spmatrix):
@@ -171,7 +176,7 @@ class ECOSSolver(Solver):
         G = self.matrix((V, (I, J)), (length, self._numVars))
 
         # Construct h.
-        constant = expression._const
+        constant = expression._constant_coef
         if not isinstance(constant, cvxopt.matrix):
             constant = cvxopt.matrix(constant)
         h = self.array(constant, dtype=float).flatten()
@@ -291,20 +296,16 @@ class ECOSSolver(Solver):
             -self.stack(Gz, Gx, Gy), self.stack(hz, hx, hy), "e")
 
     def _import_objective(self):
-        direction, objective = self.ext.objective
+        direction, objective = self.ext.no
 
-        # ECOS does not support a constant offset.
-        # self._objectiveOffset = objective._const[0]
+        # ECOS only supports minimization; flip the sign for maximization.
+        sign = 1.0 if direction == "min" else -1.0
 
-        if direction != "find":
-            # ECOS only supports minimization; flip the sign for maximization.
-            sign = 1.0 if direction == "min" else -1.0
-
-            # Import coefficients.
-            for variable, coefficient in objective._coefs.items():
-                for localIndex in range(variable.dim):
-                    index = self._ecosVarOffset[variable] + localIndex
-                    self.int["c"][index] = sign * coefficient[localIndex]
+        # Import coefficients.
+        for variable, coefficient in objective._linear_coefs.items():
+            for localIndex in range(variable.dim):
+                index = self._ecosVarOffset[variable] + localIndex
+                self.int["c"][index] = sign * coefficient[localIndex]
 
     def _import_problem(self):
         self._numVars = sum(var.dim for var in self.ext.variables.values())
@@ -370,8 +371,8 @@ class ECOSSolver(Solver):
 
         # Make sure that no unsupported constraints are present.
         for constraint in self.ext.constraints.values():
-            assert isinstance(constraint, (AffineConstraint, SOCConstraint,
-                RSOCConstraint, ExpConeConstraint))
+            assert isinstance(constraint, (DummyConstraint, AffineConstraint,
+                SOCConstraint, RSOCConstraint, ExpConeConstraint))
 
         # Set objective.
         self._import_objective()
@@ -457,6 +458,11 @@ class ECOSSolver(Solver):
         duals = {}
         if self.ext.options.duals is not False:
             for constraint in self.ext.constraints.values():
+                if isinstance(constraint, DummyConstraint):
+                    duals[constraint] = cvxopt.spmatrix(
+                        [], [], [], constraint.size)
+                    continue
+
                 dual    = None
                 indices = self._ecosConIndices[constraint]
 
@@ -489,25 +495,28 @@ class ECOSSolver(Solver):
 
                 duals[constraint] = dual
 
-        # # Retrieve objective value.
-        # p = solution["info"]["pcost"]
-        # d = solution["info"]["dcost"]
-        #
-        # if p is not None and d is not None:
-        #     objectiveValue = 0.5 * (p + d)
-        # elif p is not None:
-        #     objectiveValue = p
-        # elif d is not None:
-        #     objectiveValue = d
-        # else:
-        #     objectiveValue = None
-        #
-        # if objectiveValue is not None:
-        #     objectiveValue += self._objectiveOffset
-        #
-        #     if self.ext.objective.direction == "max":
-        #         objectiveValue = -objectiveValue
+        # Retrieve objective value.
+        p = solution["info"]["pcost"]
+        d = solution["info"]["dcost"]
 
+        if p is not None and d is not None:
+            value = 0.5 * (p + d)
+        elif p is not None:
+            value = p
+        elif d is not None:
+            value = d
+        else:
+            value = None
+
+        if value is not None:
+            # Add back the constant part.
+            value += self.ext.no.function._constant_coef[0]
+
+            # Flip back the sign for maximization.
+            if self.ext.no.direction == "max":
+                value = -value
+
+        # Retrieve solution status.
         statusCode = solution["info"]["exitFlag"]
         if   statusCode == 0:  # optimal
             primalStatus   = SS_OPTIMAL
@@ -558,8 +567,8 @@ class ECOSSolver(Solver):
             dualStatus     = SS_UNKNOWN
             problemStatus  = PS_UNKNOWN
 
-        return self._make_solution(primals, duals, primalStatus, dualStatus,
-            problemStatus, {"ecos_sol": solution})
+        return self._make_solution(value, primals, duals, primalStatus,
+            dualStatus, problemStatus, {"ecos_sol": solution})
 
 
 # --------------------------------------
