@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2017-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`GLPKSolver`."""
@@ -24,8 +21,7 @@
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
-from ..constraints import AffineConstraint
+from ..constraints import AffineConstraint, DummyConstraint
 from ..expressions import AffineExpression, BinaryVariable, IntegerVariable
 from ..modeling.footprint import Specification
 from ..modeling.solution import (PS_FEASIBLE, PS_INFEASIBLE, PS_UNBOUNDED,
@@ -40,21 +36,27 @@ _API_START = api_start(globals())
 class GLPKSolver(Solver):
     """Interface to the GLPK solver via swiglpk."""
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -64,13 +66,18 @@ class GLPKSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("swiglpk")
+        cls.check_import("swiglpk")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "glpk", "GLPK", "GNU Linear Programming Kit"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return True
+
     def __init__(self, problem):
         """Initialize a GLPK solver interface.
 
@@ -255,10 +262,10 @@ class GLPKSolver(Solver):
             glpk.glp_set_obj_dir(p, glpk.GLP_MAX)
 
         # Set objective function shift (index 0).
-        glpk.glp_set_obj_coef(p, 0, objective._const[0])
+        glpk.glp_set_obj_coef(p, 0, objective._constant_coef[0])
 
         # Set objective function coefficient of the scalar variable.
-        for picosVar, picosCoef in objective._coefs.items():
+        for picosVar, picosCoef in objective._linear_coefs.items():
             for localIndex in range(picosVar.dim):
                 if picosCoef[localIndex]:
                     glpkIndex = self._glpkVarOffset[picosVar] + localIndex
@@ -291,7 +298,8 @@ class GLPKSolver(Solver):
 
         # Import constraints.
         for constraint in self.ext.constraints.values():
-            self._import_constraint(constraint)
+            if not isinstance(constraint, DummyConstraint):
+                self._import_constraint(constraint)
 
         # Set objective.
         self._import_objective()
@@ -506,6 +514,11 @@ class GLPKSolver(Solver):
         duals = {}
         if self.ext.options.duals is not False and continuous:
             for constraint in self.ext.constraints.values():
+                if isinstance(constraint, DummyConstraint):
+                    duals[constraint] = cvxopt.spmatrix(
+                        [], [], [], constraint.size)
+                    continue
+
                 value = []
 
                 for localIndex in range(len(constraint)):
@@ -526,14 +539,15 @@ class GLPKSolver(Solver):
 
                 duals[constraint] = dual
 
-        # # Retrieve objective value.
-        # if simplex:
-        #     objectiveValue = glpk.glp_get_obj_val(p)
-        # elif interior:
-        #     objectiveValue = glpk.glp_ipt_obj_val(p)
-        # else:
-        #     objectiveValue = glpk.glp_mip_obj_val(p)
+        # Retrieve objective value.
+        if simplex:
+            value = glpk.glp_get_obj_val(p)
+        elif interior:
+            value = glpk.glp_ipt_obj_val(p)
+        else:
+            value = glpk.glp_mip_obj_val(p)
 
+        # Retrieve solution status.
         if simplex:
             probStatusCode = glpk.glp_get_status(p)
         elif interior:
@@ -624,7 +638,7 @@ class GLPKSolver(Solver):
             dualStatus = SS_EMPTY
 
         return self._make_solution(
-            primals, duals, prmlStatus, dualStatus, problemStatus)
+            value, primals, duals, prmlStatus, dualStatus, problemStatus)
 
 
 # --------------------------------------
