@@ -63,13 +63,6 @@ class GurobiSolver(Solver):
         if not result or (explain and not result[0]):
             return result
 
-        if footprint.nonconvex_quadratic_objective and footprint.continuous:
-            if explain:
-                return (False,
-                    "Continuous problems with nonconvex quadratic objectives.")
-            else:
-                return False
-
         if footprint not in cls.SUPPORTED:
             if explain:
                 return False, cls.SUPPORTED.mismatch_reason(footprint)
@@ -561,9 +554,16 @@ class GurobiSolver(Solver):
         # Handle unsupported options.
         self._handle_unsupported_option("treememory")
 
-        # Compute duals also for QPs and QCPs.
-        if self.ext.options.duals is not False and self.ext.is_continuous():
-            self.int.setParam(gurobi.GRB.Param.QCPDual, 1)
+        # Extend functionality for continuous problems.
+        if self.ext.is_continuous():
+            # Compute duals also for QPs and QC(Q)Ps.
+            if self.ext.options.duals is not False:
+                self.int.setParam(gurobi.GRB.Param.QCPDual, 1)
+
+            # Allow nonconvex quadratic objectives.
+            # TODO: Allow querying self.ext.objective directly.
+            if self.ext.footprint.nonconvex_quadratic_objective:
+                self.int.setParam(gurobi.GRB.Param.NonConvex, 2)
 
         # Attempt to solve the problem.
         with self._header(), self._stopwatch():
