@@ -28,7 +28,7 @@ from ..constraints import (AffineConstraint, ConvexQuadraticConstraint,
                            DummyConstraint, LMIConstraint, RSOCConstraint,
                            SOCConstraint)
 from ..expressions import (AffineExpression, BinaryVariable, IntegerVariable,
-                           QuadraticExpression)
+                           QuadraticExpression, SymmetricVariable)
 from ..modeling.footprint import Specification
 from ..modeling.solution import (PS_FEASIBLE, PS_ILLPOSED, PS_INFEASIBLE,
                                  PS_UNBOUNDED, PS_UNKNOWN, SS_FEASIBLE,
@@ -134,6 +134,9 @@ class MOSEKSolver(Solver):
         """Maps a PICOS (multidimensional) variable to a MOSEK scalar variable
         (start) index."""
 
+        self._mosekBarVarIndex = dict()
+        """Maps suited PICOS symmetric variables to a MOSEK bar var index."""
+
         self._mosekLinConOffset = dict()
         """Maps a PICOS (multidimensional) linear constraint to a MOSEK scalar
         constraint (start) index."""
@@ -175,6 +178,7 @@ class MOSEKSolver(Solver):
         self.int = None
 
         self._mosekVarOffset.clear()
+        self._mosekBarVarIndex.clear()
         self._mosekLinConOffset.clear()
         self._mosekQuadConIndex.clear()
         self._mosekCone.clear()
@@ -212,17 +216,97 @@ class MOSEKSolver(Solver):
 
     @staticmethod
     def _low_tri_indices(rowCount):
-        """Yield lower triangular (row, col) indices in column-major order."""
+        """Yield enumerated lower triangular indices in column-major order."""
+        lti = 0
         for col in range(rowCount):
             for row in range(col, rowCount):
-                yield (row, col)
+                yield lti, row, col
+                lti += 1
+
+    def _index_function(self, picosVar, localIndex):
+        """Support :meth:`_affinexp_pic2msk`."""
+        if picosVar in self._mosekBarVarIndex:
+            assert picosVar not in self._mosekVarOffset
+            return None
+        else:
+            return self._mosekVarOffset[picosVar] + localIndex
+
+    def _affinexp_pic2msk(self, picosExpression):
+        """Convert a PICOS affine expression to a MOSEK one.
+
+        :yield:
+            A sequence of quintuples ``(J, V, K, W, c)``, each corresponding to
+            one scalar entry of a multidimensional PICOS affine expression,
+            where ``J`` are MOSEK scalar variable indices, ``V`` are associated
+            coefficients, ``K`` are MOSEK bar var (symmetric positive
+            semidefinite variable) indices, ``W`` are indices of associated
+            coefficient matrices and where ``c`` is a scalar constant.
+        """
+        # Handle the base case (no bar vars in the problem) quickly.
+        if not self._mosekBarVarIndex:
+            for J, V, c in picosExpression.sparse_rows(self._mosekVarOffset):
+                yield J, V, (), (), c
+
+            return
+
+        barVarIndices = tuple([] for _ in range(len(picosExpression)))
+        barVarCoefs = tuple([] for _ in range(len(picosExpression)))
+
+        for var, coef in picosExpression._linear_coefs.items():
+            if var in self._mosekBarVarIndex:
+                mosekBarVarIndex = self._mosekBarVarIndex[var]
+
+                assert isinstance(var, SymmetricVariable)
+
+                n = var.shape[0]
+
+                for localIndex in range(coef.size[0]):
+                    localCoef = coef[localIndex, :]
+
+                    if not localCoef:
+                        continue
+
+                    # Devectorize local coefficient.
+                    # This works as c.T * svec(X) = <desvec(c), X> due to svec
+                    # and its inverse desvec being isometric isomorphisms.
+                    localCoefMatrix = var._vec.devectorize(localCoef.T)
+
+                    assert localCoefMatrix.size == (n, n)
+
+                    # Get lower triangular sparse rows for the coefficient.
+                    # NOTE: This is much faster than using _low_low_tri_indices.
+                    I, J, V = [], [], []
+                    for j in range(n):
+                        for i in range(j, n):
+                            v = localCoefMatrix[i, j]
+                            if v:
+                                I.append(i)
+                                J.append(j)
+                                V.append(v)
+
+                    # Create a new MOSEK symmetric coefficient matrix.
+                    # NOTE: It does not seem possible to remove these matrices
+                    #       in MOSEK 9.2 so we might leak memory whenever they
+                    #       fall out of use (e.g. objective function updates).
+                    mosekCoefMatrix = self.int.appendsparsesymmat(n, I, J, V)
+
+                    # Append bar var index and coefficient matrix index.
+                    barVarIndices[localIndex].append(mosekBarVarIndex)
+                    barVarCoefs[localIndex].append(mosekCoefMatrix)
+
+        for localIndex, (J, V, c) in enumerate(picosExpression.sparse_rows(
+                varOffsetMap=None, indexFunction=self._index_function)):
+            # Remove indices and coefficients corresponding to bar vars.
+            JV = tuple(zip(*(jv for jv in zip(J, V) if jv[0] is not None)))
+
+            J, V = JV if JV else ((),)*2
+            K, W = barVarIndices[localIndex], barVarCoefs[localIndex]
+
+            yield J, V, K, W, c
 
     def _scalar_affinexp_pic2msk(self, picosExpression):
         assert len(picosExpression) == 1
-        return picosExpression.sparse_rows(self._mosekVarOffset)[0]
-
-    def _affinexp_pic2msk(self, picosExpression):
-        return picosExpression.sparse_rows(self._mosekVarOffset)
+        return next(self._affinexp_pic2msk(picosExpression))
 
     def _quadexp_pic2msk(self, picosExpression):
         """Transform a quadratic expression from PICOS to MOSEK.
@@ -236,6 +320,10 @@ class MOSEKSolver(Solver):
         assert isinstance(picosExpression, QuadraticExpression)
         numVars = self.int.getnumvar()
 
+        # Mixing of conic and quadratic constraints is not supported by MOSEK;
+        # therefor we do not handle bar vars here.
+        assert not self._mosekBarVarIndex
+
         # Make a sparse representation of the strict lower, diagonal and strict
         # upper parts of the matrix.
         IL, JL, VL = [], [], []
@@ -312,6 +400,12 @@ class MOSEKSolver(Solver):
             self.int.putvarboundlist(
                 indices, boundKeys, lowerBounds, upperBounds)
 
+    def _import_psd_variable(self, picosVar):
+        assert isinstance(picosVar, SymmetricVariable)
+
+        self._mosekBarVarIndex[picosVar] = self.int.getnumbarvar()
+        self.int.appendbarvars([picosVar.shape[0]])
+
     def _import_linear_constraint(self, picosConstraint):
         import mosek
 
@@ -319,8 +413,6 @@ class MOSEKSolver(Solver):
         conLen  = len(picosConstraint)
         self.int.appendcons(conLen)
 
-        rows = picosConstraint.sparse_Ab_rows(self._mosekVarOffset)
-
         if picosConstraint.is_equality():
             boundKey = mosek.boundkey.fx
         elif picosConstraint.is_increasing():
@@ -328,15 +420,24 @@ class MOSEKSolver(Solver):
         else:
             boundKey = mosek.boundkey.lo
 
-        for localConIndex, (mosekVarIndices, coefs, offset) in enumerate(rows):
-            mosekConIndex = numCons + localConIndex
+        rows = self._affinexp_pic2msk(picosConstraint.lhs - picosConstraint.rhs)
 
-            self.int.putarow(mosekConIndex, mosekVarIndices, coefs)
-            self.int.putconbound(mosekConIndex, boundKey, offset, offset)
+        for localConIndex, (J, V, K, W, c) in enumerate(rows):
+            mosekConIndex = numCons + localConIndex
+            rhs = -c
+
+            self.int.putarow(mosekConIndex, J, V)
+            for k, w in zip(K, W):
+                self.int.putbaraij(mosekConIndex, k, [w], [1.0])
+            self.int.putconbound(mosekConIndex, boundKey, rhs, rhs)
 
         self._mosekLinConOffset[picosConstraint] = numCons
 
     def _import_quad_constraint(self, picosConstraint):
+        # Mixing of conic and quadratic constraints is not supported by MOSEK;
+        # therefor we do not handle bar vars here.
+        assert not self._mosekBarVarIndex
+
         # Import the linear part first.
         picosLinConPart = picosConstraint.le0.aff < 0
         self._import_linear_constraint(picosLinConPart)
@@ -373,14 +474,14 @@ class MOSEKSolver(Solver):
             entries.append(self._scalar_affinexp_pic2msk(f*picosConstraint.ub2))
         else:
             entries.append(self._scalar_affinexp_pic2msk(picosConstraint.ub))
-        entries += self._affinexp_pic2msk(picosConstraint.ne)
+        entries.extend(self._affinexp_pic2msk(picosConstraint.ne))
 
         # Map cone member entries to existing MOSEK variables, if possible.
         mosekVarsMissing = []
-        for scalarVarNum, (indices, values, offset) in enumerate(entries):
-            if len(values) == 1 and values[0] == 1.0 and offset == 0.0 \
-            and not self._var_was_used_in_cone(indices[0], mosekVars):
-                mosekVars.append(indices[0])
+        for scalarVarNum, (J, V, K, W, c) in enumerate(entries):
+            if len(J) == 1 and V[0] == 1.0 and not K and not W and not c \
+            and not self._var_was_used_in_cone(J[0], mosekVars):
+                mosekVars.append(J[0])
             else:
                 mosekVars.append(None)
                 mosekVarsMissing.append(scalarVarNum)
@@ -398,34 +499,26 @@ class MOSEKSolver(Solver):
 
         # Constrain the auxiliary variables to be equal to the cone member
         # entries for which no existing MOSEK variable could be used.
+        # TODO: Instead of always creating a constraint, fix variables via their
+        #       bound whenever possible.
         for auxNum, missingVarIndex in enumerate(mosekVarsMissing):
             auxVarIndex = auxVarOffset + auxNum
             auxConIndex = auxConOffset + auxNum
 
             # Prepare the auxiliary constraint.
-            indices, values, offset = entries[missingVarIndex]
-
-            # TODO: Instead of always creating a constraint, fix variables via
-            #       their bound whenever possible.
-            # if len(indices) is 0:
-            #     if self._debug():
-            #         self._debug("  Fixing MOSEK auxiliary variable: "
-            #                 "x[{}] = {}".format(auxVarIndex, offset))
-            #
-            #     self.int.putvarbound(
-            #         auxVarIndex, mosek.boundkey.fx, offset, offset)
-            # else:
-            indices.append(auxVarIndex)
-            values.append(-1.0)
+            J, V, K, W, c = entries[missingVarIndex]
 
             if self._debug():
                 self._debug("  Adding MOSEK auxiliary constraint: "
-                    "{}.T * x{} = {}".format(values, indices, -offset))
+                    "{}.T * x{}{} = {}"
+                    .format(V, J, " + f(barvars)" if K else "", -c))
 
             # Add the auxiliary constraint.
-            self.int.putarow(auxConIndex, indices, values)
-            self.int.putconbound(
-                auxConIndex, mosek.boundkey.fx, -offset, -offset)
+            self.int.putarow(auxConIndex, J, V)
+            self.int.putaij(auxConIndex, auxVarIndex, -1.0)
+            for k, w in zip(K, W):
+                self.int.putbaraij(auxConIndex, k, [w], [1.0])
+            self.int.putconbound(auxConIndex, mosek.boundkey.fx, -c, -c)
 
             # Complete the mapping of cone member entries to MOSEK (auxiliary)
             # variables (and auxiliary constraints).
@@ -444,6 +537,8 @@ class MOSEKSolver(Solver):
         self._mosekCone[picosConstraint] = (coneIndex, mosekVars, mosekCons)
 
     def _import_sdp_constraint(self, picosConstraint):
+        # NOTE: Trivial LMIs of the form X ≽ 0 are loaded as bar vars instead.
+
         import mosek
 
         n = picosConstraint.size[0]
@@ -470,31 +565,32 @@ class MOSEKSolver(Solver):
             Units = self._mosekBarUnitCoefs[n] = [
                 self.int.appendsparsesymmat(
                     n, [row], [col], [1.0 if row == col else 0.5])
-                for row, col in self._low_tri_indices(n)]
+                for col in range(n) for row in range(col, n)]
 
         # We iterate over the lower triangular scalar sub-expressions of the
         # expression that the PICOS constraint states to be PSD, and constrain
         # them to be eqal to the MOSEK "bar var" at the same index.
-        psd = picosConstraint.psd
-        psdRows = psd.sparse_rows(self._mosekVarOffset, lowerTriangle=True)
-        for lowTriIndex, (row, col) in enumerate(self._low_tri_indices(n)):
+        psdRows = tuple(self._affinexp_pic2msk(picosConstraint.psd))
+        for lowTriIndex, row, col in self._low_tri_indices(n):
             mosekConIndex = mosekConOffset + lowTriIndex
-            indices, coefficients, offset = psdRows[lowTriIndex]
+            J, V, K, W, c = psdRows[row + n*col]
+            rhs = -c
 
             # The lower-triangular entries in the PSD-constrained matrix …
-            self.int.putarow(mosekConIndex, indices, coefficients)
+            self.int.putarow(mosekConIndex, J, V)
+            for k, w in zip(K, W):
+                self.int.putbaraij(mosekConIndex, k, [w], [1.0])
 
             # … minus the corresponding bar var entries …
             self.int.putbaraij(
                 mosekConIndex, barVar, [Units[lowTriIndex]], [-1.0])
 
             # … should equal zero.
-            self.int.putconbound(
-                mosekConIndex, mosek.boundkey.fx, -offset, -offset)
+            self.int.putconbound(mosekConIndex, mosek.boundkey.fx, rhs, rhs)
 
             if self._debug():
-                self._debug("  Index {} ({}, {}): indices = {}, coefs = {}"
-                    .format(lowTriIndex, row, col, indices, coefficients))
+                self._debug("  Index {} ({}, {}): J = {}, V = {}, ..."
+                    .format(lowTriIndex, row, col, J, V))
 
         self._mosekLMI[picosConstraint] = (barVar, mosekConOffset)
 
@@ -517,31 +613,31 @@ class MOSEKSolver(Solver):
                 picosConstraint.__class__.__name__)
 
     def _reset_objective(self):
-        # Reset affine part.
         numVars = self.int.getnumvar()
+        numBarVars = self.int.getnumbarvar()
+
+        # Reset affine part.
         self.int.putclist(range(numVars), [0.0]*numVars)
+        for k in range(numBarVars):
+            self.int.putbarcj(k, [], [])
         self.int.putcfix(0.0)
 
         # Reset quadratic part.
         self.int.putqobj([], [], [])
 
     def _import_affine_objective(self, picosObjective):
-        mosekIndices      = []
-        mosekCoefficients = []
+        J, V, K, W, c = self._scalar_affinexp_pic2msk(picosObjective)
 
-        for picosVar, picosCoef in picosObjective._linear_coefs.items():
-            for localIndex in range(picosVar.dim):
-                if picosCoef[localIndex]:
-                    mosekIndex = self._mosekVarOffset[picosVar] + localIndex
-                    mosekIndices.append(mosekIndex)
-                    mosekCoefficients.append(picosCoef[localIndex])
-
-        self.int.putclist(mosekIndices, mosekCoefficients)
-
-        if picosObjective._constant_coef:
-            self.int.putcfix(picosObjective._constant_coef[0])
+        self.int.putclist(J, V)
+        for k, w in zip(K, W):
+            self.int.putbarcj(k, [w], [1.0])
+        self.int.putcfix(c)
 
     def _import_quadratic_objective(self, picosObjective):
+        # Mixing of conic and quadratic constraints is not supported by MOSEK;
+        # therefor we do not handle bar vars here.
+        assert not self._mosekBarVarIndex
+
         # Import the quadratic part.
         self.int.putqobj(*self._quadexp_pic2msk(picosObjective))
 
@@ -571,12 +667,26 @@ class MOSEKSolver(Solver):
         # Create a problem instance.
         self.int = self.env.Task()
 
-        # Import variables.
-        for variable in self.ext.variables.values():
-            self._import_variable(variable)
-
-        # Import constraints.
+        # Convert trivial LMIs and their symmetric variable into a PSD variable.
+        # This allows the constraint to be loaded more efficiently.
+        constraints, psdVars = [], set()
         for constraint in self.ext.constraints.values():
+            if isinstance(constraint, LMIConstraint) and constraint.semidefVar:
+                psdVars.add(constraint.semidefVar)
+            else:
+                constraints.append(constraint)
+
+        # Import normal variables.
+        for variable in self.ext.variables.values():
+            if variable not in psdVars:
+                self._import_variable(variable)
+
+        # Import virtual PSD variables as MOSEK "bar variables".
+        for variable in psdVars:
+            self._import_psd_variable(variable)
+
+        # Import remaining constraints.
+        for constraint in constraints:
             self._import_constraint(constraint)
 
         # Set objective.
@@ -612,6 +722,7 @@ class MOSEKSolver(Solver):
 
         # Reset all solver options to default.
         self.int.setdefaults()
+        self.int.putoptserverhost("")
 
         # verbosity
         if self._verbose():
@@ -724,6 +835,10 @@ class MOSEKSolver(Solver):
             except mosek.Error as error:
                 self._handle_bad_solver_specific_option(key, value, error)
 
+        # Handle 'mosek_server' option.
+        if self.ext.options.mosek_server:
+            self.int.putoptserverhost(self.ext.options.mosek_server)
+
         # Handle unsupported options.
         # TODO: Handle "hotstart" option (via mio_construct_sol).
         self._handle_unsupported_option("hotstart", "treememory")
@@ -760,8 +875,34 @@ class MOSEKSolver(Solver):
             self.int.getxx(solType, values)
 
             for picosVar in self.ext.variables.values():
-                mosekOffset = self._mosekVarOffset[picosVar]
-                primal = values[mosekOffset:mosekOffset + picosVar.dim]
+                if picosVar in self._mosekBarVarIndex:
+                    barVarIndex = self._mosekBarVarIndex[picosVar]
+
+                    assert isinstance(picosVar, SymmetricVariable)
+
+                    n, d = picosVar.shape[0], picosVar.dim
+                    lowTriPrimal = [float("nan")]*d
+                    matrixPrimal = [float("nan")]*n**2
+
+                    # Obtain a lower triangular vectorization from MOSEK.
+                    self.int.getbarxj(solType, barVarIndex, lowTriPrimal)
+
+                    # Convert to a full vectorization.
+                    # TODO: Immediately convert to a symmetric vectorization.
+                    for lti, row, col in self._low_tri_indices(n):
+                        value = lowTriPrimal[lti]
+                        matrixPrimal[n*row + col] = value
+                        if row != col:
+                            matrixPrimal[n*col + row] = value
+
+                    # Load the full vectorization as a CVXOPT matrix.
+                    matrixPrimal = cvxopt.matrix(matrixPrimal, (n, n))
+
+                    # Re-vectorize using a symmetric vectorization.
+                    primal = picosVar._vec.vectorize(matrixPrimal)
+                else:
+                    mosekOffset = self._mosekVarOffset[picosVar]
+                    primal = values[mosekOffset:mosekOffset + picosVar.dim]
 
                 if float("nan") in primal:
                     primals[picosVar] = None
@@ -805,11 +946,21 @@ class MOSEKSolver(Solver):
                         dual[1] = -dual[1] / math.sqrt(2.0)
                         dual[2:] = [-du for du in dual[2:]]
                 elif isinstance(constraint, LMIConstraint):
+                    if constraint.semidefVar:
+                        assert constraint not in self._mosekLMI
+                        picosVar = constraint.semidefVar
+                        assert picosVar in self._mosekBarVarIndex
+                        barVar = self._mosekBarVarIndex[picosVar]
+                    else:
+                        assert not constraint.semidefVar
+                        assert constraint in self._mosekLMI
+                        barVar, _ = self._mosekLMI[constraint]
+
                     n = constraint.size[0]
-                    barVar, _ = self._mosekLMI[constraint]
-                    lowerTriangularDual = [float("nan")]*(n*(n + 1) // 2)
+                    d = n*(n + 1) // 2
+                    lowerTriangularDual = [float("nan")]*d
                     self.int.getbarsj(solType, barVar, lowerTriangularDual)
-                    for lti, (row, col) in enumerate(self._low_tri_indices(n)):
+                    for lti, row, col in self._low_tri_indices(n):
                         value = lowerTriangularDual[lti]
                         dual[n*row + col] = value
                         if row != col:
